library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity HyperBus_bench is
end HyperBus_bench;

architecture Behavioral of HyperBus_bench is

component s27kl0641 is port(
    DQ7: inout std_logic;
    DQ6: inout std_logic;
    DQ5: inout std_logic;
    DQ4: inout std_logic;
    DQ3: inout std_logic;
    DQ2: inout std_logic;
    DQ1: inout std_logic;
    DQ0: inout std_logic;
    RWDS: inout std_logic;
    CSNeg: in std_logic;
    CK: in std_logic;
    RESETNeg: in std_logic);
end component;

component s27ks0641 is port(
    DQ7: inout std_logic;
    DQ6: inout std_logic;
    DQ5: inout std_logic;
    DQ4: inout std_logic;
    DQ3: inout std_logic;
    DQ2: inout std_logic;
    DQ1: inout std_logic;
    DQ0: inout std_logic;
    RWDS: inout std_logic;
    CSNeg: in std_logic;
    CK: in std_logic;
	CKNeg: in std_logic;
    RESETNeg: in std_logic);
end component;

component IS66WVH16M8ALL is port(
    ck: in std_logic;
    ckb: in std_logic;
    psc: in std_logic;
    pscb: in std_logic;
    csb: in std_logic;
    resetb: in std_logic;
    dq: inout std_logic_vector(7 downto 0);
    rwds: inout std_logic);
end component;

component HyperBus is Port(
	clk100: in std_logic;
	start: in std_logic;
	mem_regs: in std_logic; --0: Mem, 1: Regs
	burstlength: in std_logic; --0: 2Bytes, 1:64Bytes
	adr: in std_logic_vector(21 downto 0); -- 21 ... 9: 8192 Rows, 8 ... 0: 512 Words in 1 Row. (2 ... 0: 8 Words = 1 Half Page)
	snd_rcv: in std_logic;
	ready: out std_logic;
	--- SND ---
	data_snd: in std_logic_vector(64*8-1 downto 0);
	--- RCV --
	data_rcv_valid: out std_logic;
	data_rcv: out std_logic_vector(64*8-1 downto 0);
	--- RAM ---
	nReset: out std_logic;
	nCS: out std_logic;
	CK: out std_logic;
	nCK: out std_logic;
	PSC: out std_logic;
	nPSC: out std_logic;
	RWDS: inout std_logic;
	DQ: inout std_logic_vector(7 downto 0));
end component;

signal clk100: std_logic:='0';
signal start: std_logic:='0';
signal snd_rcv: std_logic:='0';
signal mem_regs: std_logic:='0';
signal burstlength: std_logic:='0';
signal adr: std_logic_vector(21 downto 0):=(others => '0');
signal ready: std_logic:='0';
signal data_snd: std_logic_vector(64*8-1 downto 0):=x"A332A331A330A329A328A327A326A325A224A223A222A221A220A219A218A217A116A115A114A113A112A111A110A109A008A007A006A005A004A003A002A001";
signal data_rcv: std_logic_vector(64*8-1 downto 0):=(others => '0');
signal data_rcv_valid: std_logic:='0';
signal nReset: std_logic:='0';
signal nCS: std_logic:='0';
signal CK: std_logic:='0';
signal nCK: std_logic:='0';
signal PSC: std_logic:='0';
signal nPSC: std_logic:='0';
signal RWDS: std_logic:='1';
signal DQ: std_logic_vector(7 downto 0):="ZZZZZZZZ";

begin
	
--RAM: s27kl0641 port map(
--    DQ7 => DQ(7),
--    DQ6 => DQ(6),
--    DQ5 => DQ(5),
--    DQ4 => DQ(4),
--    DQ3 => DQ(3),
--    DQ2 => DQ(2),
--    DQ1 => DQ(1),
--    DQ0 => DQ(0),
--    RWDS => RWDS,
--    CSNeg => nCS,
--    CK => CK,
--    RESETNeg => nReset);
	
--RAM: s27ks0641 port map(
--    DQ7 => DQ(7),
--    DQ6 => DQ(6),
--    DQ5 => DQ(5),
--    DQ4 => DQ(4),
--    DQ3 => DQ(3),
--    DQ2 => DQ(2),
--    DQ1 => DQ(1),
--    DQ0 => DQ(0),
--    RWDS => RWDS,
--    CSNeg => nCS,
--    CK => CK,
--	CKNeg => nCK,
--    RESETNeg => nReset);
	
RAM: IS66WVH16M8ALL port map(
    ck => CK,
    ckb => nCK,
    psc => PSC,
    pscb => nPSC,
    csb => nCS,
    resetb => nReset,
    dq => DQ,
    rwds => RWDS);

UUT: HyperBus port map(
	clk100 => clk100,
	start => start,
	mem_regs => mem_regs,
	burstlength => burstlength,
	adr => adr,
	snd_rcv => snd_rcv,
	ready => ready,
	--- SND ---
	data_snd => data_snd,
	--- RCV --
	data_rcv_valid => data_rcv_valid,
	data_rcv => data_rcv,
	--- RAM ---
	nReset => nReset,
	nCS => nCS,
	CK => CK,
	nCK => nCK,
	PSC => PSC,
	nPSC => nPSC,
	RWDS => RWDS,
	DQ => DQ);

clk100 <= not clk100 after 5 ns;

process begin
	wait for 660 us;
	--adr <= (others => '1');
	--adr <= std_logic_vector(to_unsigned(9,22));
	--burstlength <= '0'; --SHORT
	--snd_rcv <= '0'; --WRITE
	--wait until rising_edge(clk100);
	--start <= '1';
	--wait for 10 ns;
	--start <= '0';
	----wait for 500 ns;
	
	adr <= std_logic_vector(to_unsigned(41,22));
	burstlength <= '1'; --LONG
	snd_rcv <= '0'; --WRITE
	wait until rising_edge(clk100);
	start <= '1';
	wait for 10 ns;
	start <= '0';
	
	
	wait for 2 us;
	--adr <= std_logic_vector(to_unsigned(9,22));
	--burstlength <= '0'; --SHORT
	--snd_rcv <= '1'; --READ
	--wait until rising_edge(clk100);
	--start <= '1';
	--wait for 10 ns;
	--start <= '0';
	----wait for 500 ns;
	
	adr <= std_logic_vector(to_unsigned(41,22));
	burstlength <= '1'; --LONG
	snd_rcv <= '1'; --READ
	wait until rising_edge(clk100);
	start <= '1';
	wait for 10 ns;
	start <= '0';
	wait for 1 us;
	--burstlength <= '1'; --LONG
	--snd_rcv <= '1'; --READ
	--wait until rising_edge(clk100);
	--start <= '1';
	--wait for 10 ns;
	--start <= '0';
	--wait for 1 us;
	wait;
end process;

end Behavioral;